/**
 * Single background pixel model API.
 *
 * This API allows the user to determine, be provided with the RGB values,
 * if the pixel belongs to the background or not.
 *
 * This is only an API proposition and subjected to further changes.
 */

#ifndef _BG_MODEL_H
#define _BG_MODEL_H

#define NUM_BG_EM_COMMAND 6
#define NUM_BG_K_MEANS_COMMAND 2

struct bg_commands {
	const char *bg_cmd_name;
	int len;
};

/**
 * @struct
 * used for storage data
 */
struct circle_buff {
	/** buffer for storage data */
	double *buff;
	/** number write element*/
	int head;
	/** number recorded of elements */
	int tail;
	/** buff_size - size of buffer */
	int buff_size;
	/**  dim - dimention of data */
	int dim;
};

/**
 * @function
 * new_circle_buff
 * @return pointer to struct circle_buff if OK, else return NULL
 */
struct circle_buff *new_circle_buff();

/**
 * @function
 * set_circle_buff_data_size - set data dimension and buffer size from c_buff
 * @param dim    : data dimension
 * @param ndata  : circle buffer size
 * @param c_buff : pointer to struct circle_buff
 * @note: if define DEBUG in this function done check that the c_buff is valid
 */
void set_circle_buff_data_size(int dim, int ndata, struct circle_buff *c_buff);

/**
 * @function init_circle_buff
 * initialize struct circle buff and allocates memory for struct fields
 * @param c_buff : pointer to struct circle_buff
 * @note: if define DEBUG in this function done check that the c_buff is valid
 */
void init_circle_buff(struct circle_buff *c_buff);

/**
 * @funtion clear_circle_buff
 * clear fields in struct circle buff and set data dimension and size to zero
 * @param c_buff : pointer to struct circle_buff
 * @note: if define DEBUG in this function done check that the c_buff is valid
 */
void clear_circle_buff(struct circle_buff *c_buff);

/**
 * @function
 * write_circle_buff - write data to struct circle_buff
 * @param c_buff : pointer to struct circle_buff
 * @param data   : pointer to samples
 * @note: if define DEBUG in this function done check that the c_buff and data is valid
 */
void write_circle_buff(struct circle_buff *c_buff, const double *data);

/**
 * @function
 * reset_circle_buff - set the tile and head of struct circle_buff to zero
 * @param c_buff : pointer to struct circle_buff
 * @note: if define DEBUG in this function done check that the c_buff is valid
 */
void reset_circle_buff(struct circle_buff *c_buff);

/**
 * @function
 * free memory allocated for struct circle_buff and set c_buff to NULL
 * @param c_buff : pointer to pointer to struct circle_buff
 * @note: if define DEBUG in this function done check that the c_buff is valid
 */
void free_circle_buff(struct circle_buff **c_buff);

/**
 * type of modeling pixels
 */
enum bg_colorspace {
	BG_COLORSPACE_RGB,
};

/**
 * model type
 */
enum bg_model_type {
	BG_MODEL_MOG,	 /** Mixture of Gaussians model */
	BG_MODEL_K_MEANS, /** K-means model */
};

/**
 * API functions returned values
 */
enum bg_result {
	BG_ERROR = -1,
	BG_OK,
};

/**
 * @struct
 * describing the structure of the background model
 */
struct bg_model {
	/** type of modeling pixels */
	enum bg_colorspace colorspace;
	/** model type */
	enum bg_model_type model_type;
	/** pointer to background model context */
	void *bg_model_ctx;
	/** buffer for data storage */
	struct circle_buff *buff;
};

/**
 * @function
 * bg_model_new
 * @return: pointer to struct bg_model and set type of model if OK, else NULL
 */
struct bg_model *bg_model_new(enum bg_model_type model);

/**
 * @function
 * initialize struct bg_model and allocated memory for fields of struct
 * @param bg : pointer to a struct bg_model
 * @note : if define DEBUG in this function done check that the bg is valid
 */
void bg_model_init(struct bg_model *bg);

/**
 * @function
 * clear fields of struct bg_model
 * @param bg : pointer to a struct bg_model
 * @note : if define DEBUG in this function done check that the bg is valid
 */
void bg_model_clear(struct bg_model *bg);

/**
 * @function
 * free memory allocated from struct bg_model
 * @param bg : pointer to pointer to struct bg_model
 * @note : if define DEBUG in this function done check that the bg is valid
 */
void bg_model_free(struct bg_model **bg);

/**
 * @function
 * Set colorspace. The only colorspace supported is BG_COLORSPACE_RGB.
 * @param bg: pointer to struct bg_model
 * @param colorspace: pixel colorspace
 */
void bg_model_set_colorspace(struct bg_model *bg, enum bg_colorspace colorspace);

/**
 * @function
 * Get colorspace. The only colorspace supported is BG_COLORSPACE_RGB.
 * @param bg: pointer to struct bg_model
 * @return pixel colorspace
 */
int bg_model_get_colorspace(struct bg_model *bg);

/**
 * @function
 * Set underlying model parameters using param string.
 * @param bg: pointer to struct bg_model
 * @param param: command for set model param
 * param can be: "ngaussian %d"
				 "nth mean %d %f, ..., %f",
				 "nth weight %d %f",
				 "nth matrix %d %f, ..., %f",
				 "epsilon %f"
				 "nmean %d",
		         "nth mean %d %f, .., %f"
 */
void bg_model_set_parameters(struct bg_model *bg, const char *param);

/**
 * @function
 * Set depth of the pixel value history cyclic buffer.
 * @param bg : pointer to struct bg_model
 * @param size: buffer size
 */
void bg_model_set_history_size(struct bg_model *bg, int size);

/**
 * @function
 * Get depth of the pixel value history cyclic buffer.
 * @param bg : pointer to struct bg_model
 * @return buffer size
 */
int bg_model_get_history_size(struct bg_model *bg);

/**
 * @function
 * Update the history with a new pixel value. Arguments [x,y,z] stay for
 * [R,G,B] values in BG_COLORSPACE_RGB.
 */
void bg_model_history_add(struct bg_model *bg, int x, int y, int z);

/**
 * @function
 * Reset the history and forget all previous pixel values.
 */
void bg_model_history_reset(struct bg_model *bg);

/**
 * @function
 * Acquire background model for the single pixel using it's collected historical values.
 */
void bg_model_acquire(struct bg_model *bg);

/**
 * @function
 * Update background model using a new pixel value. Arguments [x,y,z]
 * stay for [R,G,B] values in BG_COLORSPACE_RGB.
 */
void bg_model_update(struct bg_model *bg, int x, int y, int z, int n_model);

/** Reset the model to initial state. This resets underlaying mathematical model
@to initial state too. */
void bg_model_reset(struct bg_model *bg);

/**
 * @function
 * Determine, according to current background model, if a pixel [x,y,z]
 * represents background or not. Arguments [x,y,z] stay for [R,G,B] values in
 * BG_COLORSPACE_RGB
 * @return: number of model component if pixel in model, else BG_ERROR
 */
int bg_model_pixel_to_match_model(struct bg_model *bg, int x, int y, int z);

#endif	/** _BG_MODEL_H */

